/* Temporary table tests.
 *
 * Attribute list: -------------------
 * ADM-DATA 
 * AFTER-TABLE 
 * BEFORE-TABLE
 * DATA-SOURCE-MODIFIED
 * DEFAULT-BUFFER-HANDLE    *
 * DYNAMIC                  *
 * ERROR                    *
 * ERROR-STRING             *
 * HANDLE
 * HAS-RECORDS              *
 * INSTANTIATING-PROCEDURE
 * MIN-SCHEMA-MARSHAL
 * NAME                     *
 * NAMESPACE-PREFIX
 * NAMESPACE-URI
 * NO-SCHEMA-MARSHAL
 * NUM-REFERENCES
 * ORIGIN-HANDLE
 * PREPARED                 *
 * PRIMARY
 * PRIVATE-DATA
 * REJECTED 
 * SCHEMA-MARSHAL
 * SERIALIZE-NAME
 * TRACKING-CHANGES
 * TYPE                     *
 * UNDO                     *
 * UNIQUE-ID                *
 * XML-NODE-NAME
 *
 * Method list: --------------------
 * ADD-FIELDS-FROM()        *
 * ADD-INDEX-FIELD()        *
 * ADD-LIKE-FIELD()         *
 * ADD-LIKE-INDEX()         *
 * ADD-NEW-FIELD()          *
 * ADD-NEW-INDEX()          *
 * CLEAR()                  *
 * COPY-TEMP-TABLE()        *
 * CREATE-LIKE()            * 
 * CREATE-LIKE-SEQUENTIAL() - not added to progress.g
 * READ-JSON()
 * READ-XML()
 * READ-XMLSCHEMA()
 * TEMP-TABLE-PREPARE()     *
 * WRITE-JSON()
 * WRITE-XML()
 * WRITE-XMLSCHEMA()
 */
 
DISPLAY "Temporary table tests. " SKIP WITH NO-LABELS.
/* Reference table used by other temp-tables to ctreate content with like methods */
DEFINE TEMP-TABLE sample NO-UNDO 
    FIELD ch AS CHARACTER LABEL "string"
    FIELD i4 AS INTEGER   LABEL "integer" INITIAL 0
    FIELD i8 AS INT64     LABEL "long"    INITIAL 2
    FIELD l  AS LOGICAL
    INDEX idx ch ASCENDING.

/* Statically declare a temporary table, some fields are LIKE ones from 'sample' table  */
DEFINE TEMP-TABLE stt NO-UNDO 
    FIELD s-ch LIKE sample.ch INITIAL "a"
    FIELD s-dt AS DATETIME
    FIELD s-d  AS DATE LABEL "a-date"
    FIELD s-i  LIKE sample.i4 LABEL "int" INITIAL 3
    INDEX s-idx s-i DESCENDING s-d ASCENDING.
/* use a buffer handle for this table */
DEFINE VARIABLE stth AS HANDLE NO-UNDO.
stth = BUFFER stt:HANDLE.

/* Dynamically create and populate a temporary table, some fields are LIKE ones from 'sample' table  */
DEFINE VARIABLE dtth AS HANDLE NO-UNDO.

CREATE TEMP-TABLE dtth.

/* DISPLAY "NAME:" dtth:NAME SKIP.        - NAME is unknown at this time, 9040 warning is generated*/
/* DISPLAY "UNDO:" dtth:UNDO SKIP.        - UNDO is unknown at this time, 9040 warning is generated*/
/* DISPLAY "ERROR:" dtth:ERROR SKIP.      - ERROR is unknown at this time, 9040 warning is generated*/
                                 /* however ERROR-STRING is also unknown but does not generate any warnings */
/* DISPLAY "DEFAULT-BUFFER-HANDLE:" dtth:DEFAULT-BUFFER-HANDLE SKIP.    -- DEFAULT-BUFFER-HANDLE is unknown at this time, 9040 warning is generated*/

DISPLAY "dtt is now in " + STRING(dtth:PREPARED) + " state" FORMAT "x(40)" SKIP. /* should be CLEAR */ 

dtth:CREATE-LIKE("customer").             /* Copy structure of database table*/
dtth:ADD-FIELDS-FROM("customer", "id").   /* Copy fields from the same table. */
dtth:ADD-FIELDS-FROM(stth).               /* Copy fields from other (temp) table. */
dtth:CLEAR().                             /* I changed my mind */

dtth:CREATE-LIKE("sample").    /*Copy structure of sample temp-table previously defined */
dtth:TEMP-TABLE-PREPARE("dtth").
dtth:COPY-TEMP-TABLE(stth, FALSE, FALSE, TRUE, "duplicate").
dtth:CLEAR().                  /* I changed my mind, again */

DISPLAY "dtth is now in " + STRING(dtth:PREPARED) + " state" FORMAT "x(40)" SKIP. /* should be UNPREPARED */ 
DISPLAY "Preparing the table" SKIP.       /* !!! some values might not be correct at runtime, we are interested in conversion now !!!*/

/* create a new field */
dtth:ADD-NEW-FIELD("d-i8", "int64").
dtth:ADD-NEW-FIELD("d-char", "character", 2, "x(10)", "initial", "label", "column-label"). /* array of 2 strings */

/* create a new index */
dtth:ADD-NEW-INDEX("d-idx1").
dtth:ADD-NEW-INDEX("d-idx2", true, false, false).

/* After adding/declaring index's name, declare its content */
dtth:ADD-INDEX-FIELD("d-idx2", "d-i8", false).        /* add i8 as descending component of the index idx2 */
dtth:ADD-INDEX-FIELD("d-idx2", "d-char").             /* add char as ascending (default) component of the index idx2 */

/* Add field like sample.ch */
dtth:ADD-LIKE-FIELD("d-ch", "sample.ch").
dtth:ADD-LIKE-FIELD("d-i4", "sample.i4").
dtth:ADD-LIKE-FIELD(stth:NAME, stth).

dtth:ADD-LIKE-INDEX("d-idx3","name","Customer").

dtth:TEMP-TABLE-PREPARE("dtth").
DISPLAY "dtth is now in " + STRING(dtth:PREPARED) + " state" FORMAT "x(40)" SKIP. /* should be PREPARED */ 


/* test NAME property */
DISPLAY "Table name: " dtth:NAME.
dtth:NAME = "hddt".
DISPLAY ". Changed to: " dtth:NAME SKIP.

/* test UNDO property */
DISPLAY "Table can undo: " dtth:UNDO.
dtth:UNDO = TRUE.
DISPLAY ". Changed to: " dtth:UNDO SKIP.

/* test ERROR property */
DISPLAY "Table ERROR: " dtth:ERROR.
dtth:ERROR = TRUE.
DISPLAY ". Changed to: " dtth:ERROR SKIP.

/* test ERROR-STRING property */
DISPLAY "Table ERROR-STRING: " dtth:ERROR-STRING.
dtth:ERROR-STRING = "no error".
DISPLAY ". Changed to: " dtth:ERROR-STRING SKIP.

/* test R/O property */
DISPLAY "Table TYPE: "          stth:TYPE           dtth:TYPE           SKIP.
DISPLAY "Table UNIQUE-ID:"      stth:UNIQUE-ID      dtth:UNIQUE-ID      SKIP.
DISPLAY "Table DYNAMIC:"        stth:DYNAMIC        dtth:DYNAMIC        SKIP.
DISPLAY "Table HAS-RECORDS:"                        dtth:HAS-RECORDS    SKIP.        /* HAS-RECORDS is not a queryable attribute for BUFFER widget. (4052) */
DISPLAY "Table DEFAULT-BUFFER-HANDLE:"              dtth:DEFAULT-BUFFER-HANDLE SKIP. /* DEFAULT-BUFFER-HANDLE is not a queryable attribute for BUFFER widget. (4052) */

DELETE OBJECT dtth.  /* temp-table cleanup */


